#include <math.h>

#include <qapp.h>
#include <qpainter.h>
#include <qcolor.h>

#include "newton1.h"
//  #include <qpixmap.h>

Newton1::Newton1( QWidget* parent=0, const char* name=0 )
  : QWidget( parent, name )
{
  setCaption( "Convergence map" );

  m_fdim = -1.0;
  
  m_iMax = 20;
  m_epsilon = 0.001;

  m_conv = 0;
  m_pix = 0;

  m_interrupt = 0;
  m_selecting = 0;

  setVisualDomain( 300, 300 );
  setComputationDomain( -2.0, -2.0, 2.0, 2.0 );
}

Newton1::~Newton1()
{
  delete m_conv;
  delete m_pix;
}

QSize Newton1::sizeHint()
{
  return( QSize( m_W, m_H ) );
}

void Newton1::paintEvent( QPaintEvent * )
{
  Int32 x0, y0, x1, y1;

  if (m_selecting) {
    x0 = QMIN( m_xS0, m_xS1o );
    x1 = QMAX( m_xS0, m_xS1o );
    y0 = QMIN( m_yS0, m_yS1o );
    y1 = QMAX( m_yS0, m_yS1o );
    bitBlt( this, x0, y0, m_pix, x0, y0, x1 - x0 + 1, y1 - y0 + 1 );
    QPainter p( this );
    p.setPen( black );
    p.drawRect( m_xS0, m_yS0, m_xS1 - m_xS0 + 1, m_yS1 - m_yS0 + 1 );
  }
  else bitBlt( this, 0, 0, m_pix, 0, 0, m_W, m_H );
}

void Newton1::mousePressEvent( QMouseEvent* e )
{
  sendCoordinates( e->x(), e->y() );
  if (e->button() == RightButton) {
    m_selecting = 1;
    m_xS1 = m_xS1o = m_xS0 = e->x();
    m_yS1 = m_yS1o = m_yS0 = e->y();
    repaint( FALSE );
  }
}

void Newton1::mouseMoveEvent( QMouseEvent* e )
{
  sendCoordinates( e->x(), e->y() );
  if (e->state() == RightButton) {
    m_xS1o = m_xS1;
    m_yS1o = m_yS1;
    m_xS1 = e->x();
    m_yS1 = e->y();
    repaint( FALSE );
  }
}

void Newton1::mouseReleaseEvent( QMouseEvent* e )
{
  Float x0, y0, x1, y1, bx0, by0, bx1, by1;
  if (e->button() == RightButton) {
    m_selecting = 0;
    repaint( FALSE );
    if ((m_xS1 != m_xS0) && (m_yS1 != m_yS0)) {
      bx0 = QMIN( m_xC0, m_xC1 );
      by0 = QMIN( m_yC0, m_yC1 );
      bx1 = QMAX( m_xC0, m_xC1 );
      by1 = QMAX( m_yC0, m_yC1 );
      x0 = ((CONVF (m_xS0 - m_xV0) / (m_xV1 - m_xV0)) *
	    (m_xC1 - m_xC0) + m_xC0);
      if (x0 < bx0) x0 = bx0;
      if (x0 > bx1) x0 = bx1;
      y0 = ((CONVF (m_yV1 - m_yS0 - m_yV0) / (m_yV1 - m_yV0)) *
	    (m_yC1 - m_yC0) + m_yC0);
      if (y0 < by0) y0 = by0;
      if (y0 > by1) y0 = by1;
      x1 = ((CONVF (m_xS1 - m_xV0) / (m_xV1 - m_xV0)) *
	    (m_xC1 - m_xC0) + m_xC0);
      if (x1 < bx0) x1 = bx0;
      if (x1 > bx1) x1 = bx1;
      y1 = ((CONVF (m_yV1 - m_yS1 - m_yV0) / (m_yV1 - m_yV0)) *
	    (m_yC1 - m_yC0) + m_yC0);
      if (y1 < by0) y1 = by0;
      if (y1 > by1) y1 = by1;
      if (x0 > x1) { bx0 = x0; x0 = x1; x1 = bx0; }
      if (y0 > y1) { by0 = y0; y0 = y1; y1 = by0; }
      //      debug( "%f %f %f %f", x0, y0, x1, y1 );
      emit newComputationDomain( x0, y0, x1, y1 ); 
    }
    emit released();
    return;
  }
  if (e->button() == MidButton) {
    bx0 = QMIN( m_xC0, m_xC1 );
    by0 = QMIN( m_yC0, m_yC1 );
    bx1 = QMAX( m_xC0, m_xC1 );
    by1 = QMAX( m_yC0, m_yC1 );
    x0 = ((CONVF (e->x() - m_xV0) / (m_xV1 - m_xV0)) *
	  (m_xC1 - m_xC0) + m_xC0);
    if (x0 < bx0) x0 = bx0;
    if (x0 > bx1) x0 = bx1;
    y0 = ((CONVF (m_yV1 - e->y() - m_yV0) / (m_yV1 - m_yV0)) *
	  (m_yC1 - m_yC0) + m_yC0);
    if (y0 < by0) y0 = by0;
    if (y0 > by1) y0 = by1;
    bx1 = x0 - (m_xC1 + m_xC0) / 2.0;
    by1 = y0 - (m_yC1 + m_yC0) / 2.0; // Displacement vector.
//     debug( "%f %f %f %f %f %f",
// 	   x0, y0, bx1, by1, m_yC1 - m_yC0, m_xC1 - m_xC0 );
    x0 = m_xC0 + bx1;
    y0 = m_yC0 + by1;
    x1 = m_xC1 + bx1;
    y1 = m_yC1 + by1;
    emit newComputationDomain( x0, y0, x1, y1 ); 
  }
  emit released();
}

void Newton1::sendCoordinates( Int32 x, Int32 y )
{
  Float xx, yy, bx0, by0, bx1, by1;

  bx0 = QMIN( m_xC0, m_xC1 );
  by0 = QMIN( m_yC0, m_yC1 );
  bx1 = QMAX( m_xC0, m_xC1 );
  by1 = QMAX( m_yC0, m_yC1 );
  xx = ((CONVF (x - m_xV0) / (m_xV1 - m_xV0)) *
	(m_xC1 - m_xC0) + m_xC0);
  if (xx < bx0) xx = bx0;
  if (xx > bx1) xx = bx1;
  yy = ((CONVF (m_yV1 - y - m_yV0) / (m_yV1 - m_yV0)) *
	    (m_yC1 - m_yC0) + m_yC0);
  if (yy < by0) yy = by0;
  if (yy > by1) yy = by1;

  emit coordinates( (double) xx, (double) yy );
}

void Newton1::render()
{
  Int32 conv, ccol, xV, yV;
  Float x0, y0;
  Int32 step[5] = {16, 8, 4, 2, 1};
  Int32 fine, skip, sy, sx;
  Int32 i, colStep, ncol;
  QPainter p;
  QColor col[256];

  m_interrupt = 0;
  emit rendering();
  p.begin( m_pix );
//   col[0].setRgb( 0x00, 0x00, 0x00 );
//   col[1].setRgb( 0xff, 0xff, 0xff );
//   col[2].setRgb( 0x44, 0x44, 0x44 );
//   col[3].setRgb( 0x22, 0x22, 0x22 );
  colStep = 256 / (m_iMax);
  ncol = m_iMax + 1;
  for (i = 0; i < (m_iMax); i++) {
    col[i].setRgb( 255 - colStep*i, 255 - colStep*i, 255 - colStep*i );
    col[ncol+i].setRgb( 0, 255 - colStep*i, 0 );
    col[2*ncol+i].setRgb( 0, 0, 255 - colStep*i );
  }
  col[ncol-1].setRgb( 0, 0, 0 );
  col[2*ncol-1].setRgb( 0, 0, 0 );
  col[3*ncol-1].setRgb( 0, 0, 0 );
//   col[m_iMax+1].setRgb( 0x44, 0x44, 0x44 );
//   col[m_iMax+2].setRgb( 0x22, 0x22, 0x22 );

  for (fine = 0; fine < 5; fine++) {
//     m_pix->fill( black );
//     repaint();
    //    debug( "fine: %d, step: %d", fine, step[fine] );
    skip = (fine > 0) * 2;
    sy = 0;
    for (yV = m_yV0; yV <= m_yV1; yV += step[fine]) {
      if (skip) sy = !(yV%step[fine-1]);
      //      debug( "%d %d", yV, sy );
      sx = 0;
      y0 = ((CONVF (m_yV1 - yV - m_yV0) / (m_yV1 - m_yV0)) *
	    (m_yC1 - m_yC0) + m_yC0);
      for (xV = m_xV0; xV <= m_xV1; xV += step[fine] ) {
	if (sy) {
	  sx = !sx;
	  if (sx) continue;
	}
	x0 = ((CONVF (xV - m_xV0) / (m_xV1 - m_xV0)) *
	(m_xC1 - m_xC0) + m_xC0);
	conv = 0;
	ccol = m_iMax;
	if (!conv) {
	  ccol = iterate( x0, y0, 1.0, 0.0 );
	  conv = !(ccol == m_iMax);
	}
	if (!conv) {
	  ccol = iterate( x0, y0, -0.5, -0.86602540378 );
	  conv = !(ccol == m_iMax);
	  ccol += ncol;
	}
	if (!conv) {
	  ccol = iterate( x0, y0, -0.5, 0.86602540378 );
	  conv = !(ccol == m_iMax);
	  ccol += 2*ncol;
	}
	m_conv[m_W * yV + xV] = ccol;
	p.setPen( col[ccol] );
	if (fine < 4) {
	  p.setBrush( col[ccol] );
	  p.drawRect( xV, yV, step[fine], step[fine] );
	} else {
	  p.drawPoint( xV, yV );
	}
//	if (fine == 4) debug( "%d %d %d", xV, yV, ccol );
// 	if (conv < 1) debug( "%d %d %d", xV, yV, ccol );
// 	if (conv > m_iMax) debug( "%d %d %d", xV, yV, ccol );
      }
      repaint( FALSE );
      qApp->processEvents();
      if (m_interrupt) {
	p.end();
	emit done();
	return;
      }
    }
  }
  p.end();
  fractalDimension();
  emit done();
  m_pix->save( "picture.xpm", "XPM" );
}

Int32 Newton1::iterate( Float x0, Float y0, Float xLim, Float yLim )
{
  Float x, y, t1, t2, t3, epsX, epsY;
  Int32 i;
  
  x = x0;
  y = y0;
  epsX = m_epsilon * fabs(x0 - xLim) + m_epsilon * (x0 == xLim);
  epsY = m_epsilon * fabs(y0 - yLim) + m_epsilon * (y0 == yLim);
  i = 0;
  while (i < m_iMax) {
    t2 = (x*x - y*y);
    t3 = 2.0 * x * y;
    t1 = t2*t2 + t3*t3;
    x = (2.0 * x + t2 / t1) / 3.0;
    y = (2.0 * y - t3 / t1) / 3.0;
    i++;
    if ((fabs(x-xLim) < epsX) && (fabs(y-yLim) < epsY)) {
      return( i-1 );
    }
    /*    printf( "%d %f %f %f %f\n", i, x, y, xLim, yLim );*/
  }
  return( i );
}

void Newton1::fractalDimension()
{
  Int32 i, j, k;
  Float fdim, count, eps;

  count = 0.0;
  for (j = 1; j < (m_H-1); j++) {
    for (i = 1; i < (m_W-1); i++) {
      if (m_conv[m_W * j + i] < m_iMax) {
	k = 0;
	k += (m_conv[m_W * (j-1) + i-1] >= m_iMax);
	k += (m_conv[m_W * (j-1) + i] >= m_iMax);
	k += (m_conv[m_W * (j-1) + i+1] >= m_iMax);
	k += (m_conv[m_W * j + i-1] >= m_iMax);
	k += (m_conv[m_W * j + i+1] >= m_iMax);
	k += (m_conv[m_W * (j+1) + i-1] >= m_iMax);
	k += (m_conv[m_W * (j+1) + i] >= m_iMax);
	k += (m_conv[m_W * (j+1) + i+1] >= m_iMax);
	count += CONVF (k > 0);
      }
    }
  }
  eps = fabs( (m_yC1 - m_yC0) / (m_yV1 - m_yV0) );

  fdim = log( count ) / (fabs( log( eps ) ));

  m_fdim = fdim;
  debug( "fdim: %f", m_fdim );
}
void Newton1::setVisualDomain( Int32 w, Int32 h )
{
  delete m_conv;
  delete m_pix;

  m_W = w;
  m_H = h;

  m_xV0 = 0;
  m_yV0 = 0;
  m_xV1 = w - 1;
  m_yV1 = h - 1;

  m_conv = new Int32 [m_W * m_H];
  m_pix = new QPixmap( m_W, m_H );
  
  m_pix->fill( black );

  setFixedSize( sizeHint() );
}

void Newton1::setComputationDomain( Float x0, Float y0, Float x1, Float y1 )
{
  m_xC0 = x0;
  m_yC0 = y0;
  m_xC1 = x1;
  m_yC1 = y1;
}

void Newton1::setMaxIter( Int32 iMax )
{
  m_iMax = iMax;
}

void Newton1::setEpsilon( Float epsilon )
{
  m_epsilon = epsilon;
}

void Newton1::interrupt()
{
  m_interrupt = 1;
}

void Newton1::visualDomain( Int32 *w, Int32 *h )
{
  *w = m_W;
  *h = m_H;
}

void Newton1::computationDomain( Float *x0, Float *y0, Float *x1, Float *y1 )
{
  *x0 = m_xC0;
  *y0 = m_yC0;
  *x1 = m_xC1;
  *y1 = m_yC1;
}

Int32 Newton1::maxIter()
{
  return( m_iMax );
}

Float Newton1::epsilon()
{
  return( m_epsilon );
}

//  int main( int argc, char **argv )
//  {
//    file = fopen( "picture.xpm", "w" );
//    fprintf( file, "/* XPM */\n" );
//    fprintf( file, "static char *picture = {\n" );
//    fprintf( file, "/* width height num_colors chars_per_pixel */\n" );
//    fprintf( file, "\"%d %d %d %d\",\n", xV1 - xV0, yV1 - yV0, 4, 1 );
//    fprintf( file, "/* colors */\n" );
//    fprintf( file, "\"0 c #000000\",\n" );
//    fprintf( file, "\"1 c #ffffff\",\n" );
//    fprintf( file, "\"2 c #444444\",\n" );
//    fprintf( file, "\"3 c #222222\",\n" );
//    fprintf( file, "/* pixels */\n" );

//    for (yV = yV0; yV <= yV1; yV++) {
//      y0 = ((double) (yV - yV0) / (yV1 - yV0)) * (yC1 - yC0) + yC0;
//      fprintf( file, "\"" );
//      for (xV = xV0; xV <= xV1; xV++) {
//        x0 = ((double) (xV - xV0) / (xV1 - xV0)) * (xC1 - xC0) + xC0;
//        conv = 0;
//        if (!conv) conv = 1 * iterate( x0, y0, 1.0, 0.0 );
//        if (!conv) conv = 2 * iterate( x0, y0, -0.5, -0.86602540378 );
//        if (!conv) conv = 3 * iterate( x0, y0, -0.5, 0.86602540378 );
//        fprintf( file, "%d", conv );
//        /*      printf( "%d %d %f %f\n", xV, yV, x0, y0 ); */
//      }
//      if (yV < yV1) fprintf( file, "\",\n" );
//      else fprintf( file, "\"\n" );
//    }
//    fprintf( file, "};\n" );
//    fclose( file );

