#ifndef _NEWTON1_H_
#define _NEWTON1_H_

#include <qwidget.h>
#include <qpixmap.h>

#include "types.h"

class Newton1 : public QWidget
{
  Q_OBJECT
public:
  Newton1( QWidget* parent=0, const char* name=0 );
  ~Newton1();

  Int32 iterate( Float x0, Float y0, Float xLim, Float yLim );

  void render();
  QSize sizeHint();

  void visualDomain( Int32 *w, Int32 *h );
  void computationDomain( Float *x0, Float *y0, Float *x1, Float *y1 );
  Int32 maxIter();
  Float epsilon();

public slots:
  void setVisualDomain( Int32 w, Int32 h );
  void setComputationDomain( Float x0, Float y0, Float x1, Float y1 );
  void setMaxIter( Int32 iMax );
  void setEpsilon( Float epsilon );
  void interrupt();

signals:
  void rendering();
  void done();
  void coordinates( double f1, double f2 );
  void released();
  void newComputationDomain( double x0, double y0, double x1, double y1 ); 

protected:
  void paintEvent( QPaintEvent * );
  void mousePressEvent( QMouseEvent *e );
  void mouseMoveEvent( QMouseEvent *e );
  void mouseReleaseEvent( QMouseEvent *e );

//    void resizeEvent( QResizeEvent* );
//    void mousePressEvent( QMouseEvent *e );
//    void mouseMoveEvent( QMouseEvent *e );
  
private:
  void sendCoordinates( Int32 x, Int32 y );
  void fractalDimension();

  Int32 *m_conv;
  Float m_fdim;

  Int32 m_xV0, m_yV0, m_xV1, m_yV1, m_W, m_H;
  Float m_xC0, m_yC0, m_xC1, m_yC1;

  Int32 m_iMax;
  Float m_epsilon;

  Int32 m_interrupt;

  Int32 m_selecting;
  Int32 m_xS0, m_yS0, m_xS1, m_yS1, m_xS1o, m_yS1o;
  QPixmap *m_pix;
};

#endif
