#include <stdio.h>

#include <qapp.h>
#include <qaccel.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qgrpbox.h>
#include <qlined.h>
#include <qpushbt.h>
#include <qstring.h>

#include "wmain.h"

WMain::WMain( QWidget* parent=0, const char* name=0 )
  : QWidget( parent, name )
{
  Int32 i;
  QLabel *lab;

  setCaption( "Iterative Numerics Magic" );

  QAccel *a = new QAccel( this );
  a->connectItem( a->insertItem(Key_Q+ALT), (const QObject *) this,
                  SLOT(quit()) );

  m_inm1 = new Newton1();
  connect( m_inm1, SIGNAL( rendering() ),
	   SLOT( busy() ) );
  connect( m_inm1, SIGNAL( done() ),
	   SLOT( prepared() ) );
  connect( m_inm1, SIGNAL( coordinates( double, double ) ),
           this, SLOT( setCoordinates( double, double ) ) );
  connect( m_inm1,
	   SIGNAL( newComputationDomain( double, double, double, double ) ),
           this, SLOT( setDomain( double, double, double, double ) ) );
  connect( m_inm1, SIGNAL( released() ),
           this, SLOT( renewLabel() ) );
 		    
  QBoxLayout *laySet = new QVBoxLayout( this, 5 ); 
 
  m_labName = new QLabel( "INM Setup Control", this );
  m_labName->setFixedHeight( 2 * m_labName->sizeHint().height() );
  m_labName->setFrameStyle( QFrame::Sunken | QFrame::Panel );
  m_labName->setAlignment( AlignCenter );
  m_labName->setMargin( 10 );
  m_labName->setMinimumWidth( m_labName->sizeHint().width() );
  laySet->addWidget( m_labName );
 
  laySet->addSpacing( 5 );

  QGroupBox *grp1 = new QGroupBox( "Visual domain", this );
  grp1->setAlignment( AlignHCenter );
  laySet->addWidget( grp1 );

  QGridLayout *layEdits1 = new QGridLayout( grp1, 1, 4,
					   grp1->fontMetrics().height()+3 );
  layEdits1->setColStretch( 1, 10 );
  layEdits1->setColStretch( 3, 10 );

  for (i = 0; i < 2; i++) {
    m_led[i] = new QLineEdit( grp1 );
    m_led[i]->setMinimumHeight( m_led[i]->sizeHint().height() );
    m_led[i]->setMinimumWidth( m_led[i]->fontMetrics().width( "88888888" ) );
    connect( m_led[i], SIGNAL( textChanged( const char * ) ),
	     SIGNAL( setupChanged() ) );
    connect( m_led[i], SIGNAL( returnPressed() ), this,
	     SLOT( applyChanges() ) );
  }
  lab = new QLabel( "&a) width:", grp1 );
  lab->setMinimumSize( lab->sizeHint() );
  layEdits1->addWidget( lab, 0, 0 );
  lab->setBuddy( m_led[0] );
  layEdits1->addWidget( m_led[0], 0, 1 );

  lab = new QLabel( "&b) height:", grp1 );
  lab->setMinimumSize( lab->sizeHint() );
  layEdits1->addWidget( lab, 0, 2 );
  lab->setBuddy( m_led[1] );
  layEdits1->addWidget( m_led[1], 0, 3 );

  layEdits1->activate();

  QGroupBox *grp2 = new QGroupBox( "Computational domain", this );
  grp2->setAlignment( AlignHCenter );
  laySet->addWidget( grp2 );

  QGridLayout *layEdits2 = new QGridLayout( grp2, 2, 4,
					   grp2->fontMetrics().height()+3 );
  layEdits2->setColStretch( 1, 10 );
  layEdits2->setColStretch( 3, 10 );

  for (i = 2; i < 6; i++) {
    m_led[i] = new QLineEdit( grp2 );
    m_led[i]->setMinimumHeight( m_led[i]->sizeHint().height() );
    m_led[i]->setMinimumWidth( m_led[i]->fontMetrics().width( "88888888" ) );
    connect( m_led[i], SIGNAL( textChanged( const char * ) ),
	     SIGNAL( setupChanged() ) );
    connect( m_led[i], SIGNAL( returnPressed() ), this,
	     SLOT( applyChanges() ) );
  }
  lab = new QLabel( "&c) x0:", grp2 );
  lab->setMinimumSize( lab->sizeHint() );
  layEdits2->addWidget( lab, 0, 0 );
  lab->setBuddy( m_led[2] );
  layEdits2->addWidget( m_led[2], 0, 1 );

  lab = new QLabel( "&d) y0:", grp2 );
  lab->setMinimumSize( lab->sizeHint() );
  layEdits2->addWidget( lab, 0, 2 );
  lab->setBuddy( m_led[3] );
  layEdits2->addWidget( m_led[3], 0, 3 );

  lab = new QLabel( "&e) x1:", grp2 );
  lab->setMinimumSize( lab->sizeHint() );
  layEdits2->addWidget( lab, 1, 0 );
  lab->setBuddy( m_led[4] );
  layEdits2->addWidget( m_led[4], 1, 1 );

  lab = new QLabel( "&f) y1:", grp2 );
  lab->setMinimumSize( lab->sizeHint() );
  layEdits2->addWidget( lab, 1, 2 );
  lab->setBuddy( m_led[5] );
  layEdits2->addWidget( m_led[5], 1, 3 );

  layEdits2->activate();

  laySet->addSpacing( 5 );

  m_led[6] = new QLineEdit( this );
  m_led[6]->setMinimumHeight( m_led[6]->sizeHint().height() );
  m_led[6]->setMinimumWidth( m_led[6]->fontMetrics().width( "88888888" ) );
  connect( m_led[6], SIGNAL( textChanged( const char * ) ),
	   SIGNAL( setupChanged() ) );
  connect( m_led[6], SIGNAL( returnPressed() ), this,
	   SLOT( applyChanges() ) );

  QGridLayout *layEdits3 = new QGridLayout( 2, 2, 0 );
  laySet->addLayout( layEdits3 );

  lab = new QLabel( "&g) Maximal iterations:", this );
  lab->setMinimumSize( lab->sizeHint() );
  layEdits3->addWidget( lab, 0, 0 );
  lab->setBuddy( m_led[6] );
  layEdits3->addWidget( m_led[6], 0, 1 );

  m_led[7] = new QLineEdit( this );
  m_led[7]->setMinimumHeight( m_led[7]->sizeHint().height() );
  m_led[7]->setMinimumWidth( m_led[7]->fontMetrics().width( "88888888" ) );
  connect( m_led[7], SIGNAL( textChanged( const char * ) ),
	   SIGNAL( setupChanged() ) );
  connect( m_led[7], SIGNAL( returnPressed() ), this,
	   SLOT( applyChanges() ) );

  lab = new QLabel( "&h) Epsilon:", this );
  lab->setMinimumSize( lab->sizeHint() );
  layEdits3->addWidget( lab, 1, 0 );
  lab->setBuddy( m_led[7] );
  layEdits3->addWidget( m_led[7], 1, 1 );

  laySet->addSpacing( 5 );
  laySet->addStretch( 5 );

  QBoxLayout *layMove = new QHBoxLayout();
  laySet->addLayout( layMove );

  QPushButton *butZIn = new QPushButton( "+", this );
  butZIn->setMinimumSize( butZIn->sizeHint() );
  connect( butZIn, SIGNAL( clicked() ), this, SLOT( zoomIn() ) );
  QPushButton *butZOut = new QPushButton( "-", this );
  butZOut->setMinimumSize( butZOut->sizeHint() );
  connect( butZOut, SIGNAL( clicked() ), this, SLOT( zoomOut() ) );
  QPushButton *butUp = new QPushButton( "u", this );
  butUp->setMinimumSize( butUp->sizeHint() );
  connect( butUp, SIGNAL( clicked() ), this, SLOT( moveUp() ) );
  QPushButton *butDown = new QPushButton( "d", this );
  butDown->setMinimumSize( butDown->sizeHint() );
  connect( butDown, SIGNAL( clicked() ), this, SLOT( moveDown() ) );
  QPushButton *butRight = new QPushButton( "r", this );
  butRight->setMinimumSize( butRight->sizeHint() );
  connect( butRight, SIGNAL( clicked() ), this, SLOT( moveRight() ) );
  QPushButton *butLeft = new QPushButton( "l", this );
  butLeft->setMinimumSize( butLeft->sizeHint() );
  connect( butLeft, SIGNAL( clicked() ), this, SLOT( moveLeft() ) );
  QPushButton *butEx = new QPushButton( "ex", this );
  butEx->setMinimumSize( butEx->sizeHint() );
  connect( butEx, SIGNAL( clicked() ), this, SLOT( equalizeAxesX() ) );
  QPushButton *butEy = new QPushButton( "ey", this );
  butEy->setMinimumSize( butEy->sizeHint() );
  connect( butEy, SIGNAL( clicked() ), this, SLOT( equalizeAxesY() ) );

  layMove->addWidget( butZIn );
  layMove->addStretch( 5 );
  layMove->addWidget( butZOut );
  layMove->addStretch( 20 );
  layMove->addWidget( butDown );
  layMove->addStretch( 5 );
  layMove->addWidget( butUp );
  layMove->addStretch( 5 );
  layMove->addWidget( butLeft );
  layMove->addStretch( 5 );
  layMove->addWidget( butRight );
  layMove->addStretch( 20 );
  layMove->addWidget( butEx );
  layMove->addStretch( 5 );
  layMove->addWidget( butEy );

  QBoxLayout *layBut = new QHBoxLayout();
  laySet->addLayout( layBut );

  m_butApply = new QPushButton( " Apply ", this );
  m_butApply->setMinimumSize( m_butApply->sizeHint() );
  connect( m_butApply, SIGNAL( clicked() ), this, SLOT( applyChanges() ) );
  QPushButton *butInt = new QPushButton( " Interrupt ", this );
  butInt->setMinimumSize( butInt->sizeHint() );
  connect( butInt, SIGNAL( clicked() ), m_inm1, SLOT( interrupt() ) );
  m_butQuit = new QPushButton( " Quit ", this );
  m_butQuit->setMinimumSize( m_butQuit->sizeHint() );
  connect( m_butQuit, SIGNAL( clicked() ), this, SLOT( quit() ) );
  m_labDirty = new QLabel( "clean", this );
  m_labDirty->setFrameStyle( QFrame::Sunken | QFrame::Panel );
  m_labDirty->setAlignment( AlignCenter );
  m_labDirty->setMinimumSize( m_labDirty->sizeHint() );

  layBut->addWidget( m_butApply );
  layBut->addStretch( 10 );
  layBut->addWidget( butInt );
  layBut->addStretch( 10 );
  layBut->addWidget( m_labDirty );
  layBut->addStretch( 10 );
  layBut->addWidget( m_butQuit );
 
  laySet->activate();

  connect( this, SIGNAL( setupChanged() ), this, SLOT( setDirty() ) );

  QString s;
  Int32 ii[2];
  Float f[4];

  m_inm1->visualDomain( &ii[0], &ii[1] );
  for (i = 0; i < 2; i++) {
    s.setNum( ii[i] );
    m_led[i]->setText( (const char *) s );
  }
  m_inm1->computationDomain( &f[0], &f[1], &f[2], &f[3] );
  for (i = 0; i < 4; i++) {
    s.setNum( f[i] );
    m_led[i+2]->setText( (const char *) s );
  }
  s.setNum( m_inm1->maxIter() );
  m_led[6]->setText( (const char *) s );
  s.setNum( m_inm1->epsilon() );
  m_led[7]->setText( (const char *) s );

  m_newDomain = 0;

  resize( 0, 0 );
}

WMain::~WMain()
{
  delete m_inm1;
}

void WMain::quit()
{
  m_inm1->interrupt();
  qApp->quit();
}

void WMain::busy()
{
  m_rendering = 1;
  m_butApply->setEnabled( FALSE );
  m_butQuit->setEnabled( FALSE );
}

void WMain::prepared()
{
  m_rendering = 0;
  if (m_newDomain) {
    m_newDomain = 0;
    applyChanges();
  }
  else {
    m_butApply->setEnabled( TRUE );
    m_butQuit->setEnabled( TRUE );
  }
}

void WMain::setCoordinates( double f1, double f2 )
{
  QString s;
  s.sprintf( "x: %10.6f   y: %10.6f", f1, f2);
  m_labName->setText( (const char *) s );
}

void WMain::setDomain( double x0, double y0, double x1, double y1 )
{
  QString s;
  s.setNum( x0 );
  m_led[2]->setText( (const char *) s );
  s.setNum( y0 );
  m_led[3]->setText( (const char *) s );
  s.setNum( x1 );
  m_led[4]->setText( (const char *) s );
  s.setNum( y1 );
  m_led[5]->setText( (const char *) s );

  applyChanges();
}

void WMain::renewLabel()
{
  m_labName->setText( "INM Setup Control" );
}

void WMain::zoomIn()
{
  Int32 i;
  Float f[4], zx, zy;
  QString s;

  for (i = 0; i < 4; i++) {
    s = m_led[i+2]->text();
    f[i] = CONVF s.toFloat();
  }
  zx = ( f[2] - f[0] ) / 10.0;
  zy = ( f[3] - f[1] ) / 10.0;
  f[0] += zx;
  f[2] -= zx;
  f[1] += zy;
  f[3] -= zy;
  for (i = 0; i < 4; i++) {
    s.setNum( f[i] );
    m_led[i+2]->setText( (const char *) s );
  }
  applyChanges();
}

void WMain::zoomOut()
{
  Int32 i;
  Float f[4], zx, zy;
  QString s;

  for (i = 0; i < 4; i++) {
    s = m_led[i+2]->text();
    f[i] = CONVF s.toFloat();
  }
  zx = ( f[2] - f[0] ) / 2.0;
  zy = ( f[3] - f[1] ) / 2.0;
  f[0] -= zx;
  f[2] += zx;
  f[1] -= zy;
  f[3] += zy;
  for (i = 0; i < 4; i++) {
    s.setNum( f[i] );
    m_led[i+2]->setText( (const char *) s );
  }
  applyChanges();
}

void WMain::moveUp()
{
  moveDomain( 0.0, 20.0 );
}

void WMain::moveDown()
{
  moveDomain( 0.0, -20.0 );
}
void WMain::moveRight()
{
  moveDomain( 20.0, 0.0 );
}
void WMain::moveLeft()
{
  moveDomain( -20.0, 0.0 );
}

void WMain::moveDomain( Float dx, Float dy )
{
  Int32 i;
  Float f[4], zx, zy;
  QString s;

  for (i = 0; i < 4; i++) {
    s = m_led[i+2]->text();
    f[i] = CONVF s.toFloat();
  }
  if (dx != 0.0) zx = ( f[2] - f[0] ) / dx;
  else zx = 0.0;
  if (dy != 0.0) zy = ( f[3] - f[1] ) / dy;
  else zy = 0.0;
  f[0] += zx;
  f[2] += zx;
  f[1] += zy;
  f[3] += zy;
  for (i = 0; i < 4; i++) {
    s.setNum( f[i] );
    m_led[i+2]->setText( (const char *) s );
  }
  applyChanges();
}

void WMain::equalizeAxesX()
{
  Int32 i, ii[2];
  Float f[4], eq, center;
  QString s;

  for (i = 0; i < 2; i++) {
    s = m_led[i]->text();
    ii[i] = (Int32) s.toInt();
  }
  for (i = 0; i < 4; i++) {
    s = m_led[i+2]->text();
    f[i] = CONVF s.toFloat();
  }
  eq = (CONVF ii[1] / ii[0]) * (f[2] - f[0]) / 2.0;
  center = (f[3] + f[1]) / 2.0;
  f[3] = center + eq;
  f[1] = center - eq;
  for (i = 0; i < 4; i++) {
    s.setNum( f[i] );
    m_led[i+2]->setText( (const char *) s );
  }
  applyChanges();
}

void WMain::equalizeAxesY()
{
  Int32 i, ii[2];
  Float f[4], eq, center;
  QString s;

  for (i = 0; i < 2; i++) {
    s = m_led[i]->text();
    ii[i] = (Int32) s.toInt();
  }
  for (i = 0; i < 4; i++) {
    s = m_led[i+2]->text();
    f[i] = CONVF s.toFloat();
  }
  eq = (CONVF ii[0] / ii[1]) * (f[3] - f[1]) / 2.0;
  center = (f[2] + f[0]) / 2.0;
  f[2] = center + eq;
  f[0] = center - eq;
  for (i = 0; i < 4; i++) {
    s.setNum( f[i] );
    m_led[i+2]->setText( (const char *) s );
  }
 applyChanges();
}
 
void WMain::setDirty()
{
  m_dirty = 1;
  m_labDirty->setText( "dirty" );
  m_labDirty->setMinimumSize( m_labDirty->sizeHint() );
}

void WMain::applyChanges()
{
  QString s;
  Int32 i, ii[2];
  Float f[4];

  if (m_rendering) {
    m_newDomain = 1;
    m_inm1->interrupt();
    return;
  }
  for (i = 0; i < 2; i++) {
    s = m_led[i]->text();
    ii[i] = (Int32) s.toInt();
  }
  m_inm1->setVisualDomain( ii[0], ii[1] );
  for (i = 0; i < 4; i++) {
    s = m_led[i+2]->text();
    f[i] = CONVF s.toFloat();
  }
  m_inm1->setComputationDomain( f[0], f[1], f[2], f[3] );
  s = m_led[6]->text();
  i = (Int32) s.toInt();
  if (i > 80) i = 80;
  if (i < 1) i = 1;
  s.setNum( i );
  m_led[6]->setText( (const char *) s );
  m_inm1->setMaxIter( i );
  s = m_led[7]->text();
  m_inm1->setEpsilon( CONVF s.toFloat() );

  m_dirty = 0;
  m_labDirty->setText( "clean" );
  m_labDirty->setMinimumSize( m_labDirty->sizeHint() );

  m_inm1->setMinimumSize( m_inm1->sizeHint() );
  resize( sizeHint() );
  m_inm1->render();
}
